@@ -45,6 +45,7 @@ from barman.exceptions import (
 )
 from barman.infofile import BackupInfo, WalFileInfo
 from barman.lockfile import ConfigUpdateLock
+from barman.process import ProcessManager
 from barman.server import Server
 from barman.storage.local_file_manager import LocalFileManager
 from barman.utils import (
@@ -186,17 +187,26 @@ def argument(*name_or_flags, **kwargs):
 
 
 def command(args=None, parent=subparsers, cmd_aliases=None):
-    """Decorator to define a new subcommand in a sanity-preserving way.
+    """
+    Decorator to define a new subcommand in a sanity-preserving way.
     The function will be stored in the ``func`` variable when the parser
-    parses arguments so that it can be called directly like so::
+    parses arguments so that it can be called directly like so:
+
+    Usage example:
+    .. code-block:: python
+
         args = cli.parse_args()
         args.func(args)
-    Usage example::
+
+    .. code-block:: python
+
         @command([argument("-d", help="Enable debug mode", action="store_true")])
         def command(args):
             print(args)
-    Then on the command line::
-        $ python cli.py command -d
+
+    Then on the command line:
+    .. code:: bash
+    $ python cli.py command -d
     """
 
     if args is None:
@@ -279,6 +289,36 @@ def list_servers(args):
     output.close_and_exit()
 
 
+@command(
+    [argument("server_name", help="specifies the server name")],
+    cmd_aliases=["list-process"],
+)
+def list_processes(args=None):
+    """
+    List all the active subprocesses started by the specified server.
+    """
+    server = get_server(args)
+    proc_manager = ProcessManager(server.config)
+    processes = proc_manager.list()
+    output.result("list_processes", processes, server.config.name)
+    output.close_and_exit()
+
+
+@command(
+    [
+        argument("server_name", help="specifies the server name"),
+        argument("task", help="the task name to terminate (e.g. backup, receive-wal)"),
+    ],
+)
+def terminate_process(args):
+    """
+    Terminate a Barman server subprocess specified by task name.
+    """
+    server = get_server(args)
+    server.kill(args.task)
+    output.close_and_exit()
+
+
 @command(
     [
         argument(
@@ -506,6 +546,14 @@ def backup_completer(prefix, parsed_args, **kwargs):
             action="store_false",
             default=SUPPRESS,
         ),
+        argument(
+            "--check-timeout",
+            help="Specifies the time limit, in seconds, for the check operation. "
+            "Set to zero to disable the timeout.",
+            metavar="CHECK_TIMEOUT",
+            default=None,
+            type=check_non_negative,
+        ),
     ]
 )
 def backup(args):
@@ -555,6 +603,8 @@ def backup(args):
             )
         if hasattr(args, "bwlimit"):
             server.config.bandwidth_limit = args.bwlimit
+        if args.check_timeout is not None:
+            server.config.check_timeout = args.check_timeout
         with closing(server):
             server.backup(
                 wait=args.wait,
@@ -873,6 +923,25 @@ def rebuild_xlogdb(args):
                 "incremental backup."
             ),
         ),
+        argument(
+            "--staging-path",
+            help=(
+                "A path where intermediate files are staged during restore. When "
+                "restoring a compressed backup, it serves as a temporary location for "
+                "decompression before copying to the final destination. When restoring "
+                "an incremental backup, it is where backups are combined before "
+                "copying to the final destination. This location must have enough "
+                "space to store the decompressed/combined backup."
+            ),
+        ),
+        argument(
+            "--staging-location",
+            choices=["local", "remote"],
+            help=(
+                "Specifies whether `--staging-path` is a local or remote path. Valid"
+                "values are `local` and `remote`."
+            ),
+        ),
         argument(
             "--recovery-conf-filename",
             dest="recovery_conf_filename",
@@ -981,6 +1050,44 @@ def restore(args):
 
         backup_info = server.get_backup(backup_id)
 
+    # Parse and overrides the staging path if set
+    if args.staging_path is not None:
+        try:
+            staging_path = parse_staging_path(args.staging_path)
+        except ValueError as exc:
+            output.error("Cannot parse staging path: %s", str(exc))
+            output.close_and_exit()
+        server.config.staging_path = staging_path
+
+    # Parse and overrides the staging location if set
+    if args.staging_location is not None:
+        if args.staging_location == "remote" and not args.remote_ssh_command:
+            output.error(
+                "--staging-location as remote requires --remote-ssh-command to be set"
+            )
+            output.close_and_exit()
+        server.config.staging_location = args.staging_location
+
+    # TODO: remove once we remove the deprecated recovery_staging_path option
+    # Parse and overrides the recovery staging path if set
+    if args.recovery_staging_path is not None:
+        try:
+            recovery_staging_path = parse_staging_path(args.recovery_staging_path)
+        except ValueError as exc:
+            output.error("Cannot parse recovery staging path: %s", str(exc))
+            output.close_and_exit()
+        server.config.recovery_staging_path = recovery_staging_path
+
+    # TODO: remove once we remove the deprecated local_staging_path option
+    # Parse and overrides the local staging path if set
+    if args.local_staging_path is not None:
+        try:
+            local_staging_path = parse_staging_path(args.local_staging_path)
+        except ValueError as exc:
+            output.error("Cannot parse local staging path: %s", str(exc))
+            output.close_and_exit()
+        server.config.local_staging_path = local_staging_path
+
     if backup_info.status not in BackupInfo.STATUS_COPY_DONE:
         output.error(
             "Cannot restore from backup '%s' of server '%s': "
@@ -990,58 +1097,84 @@ def restore(args):
         )
         output.close_and_exit()
 
-    # If the backup to be recovered is compressed then there are additional
-    # checks to be carried out
-    if backup_info.compression is not None:
-        # Set the recovery staging path from the cli if it is set
-        if args.recovery_staging_path is not None:
-            try:
-                recovery_staging_path = parse_staging_path(args.recovery_staging_path)
-            except ValueError as exc:
-                output.error("Cannot parse recovery staging path: %s", str(exc))
-                output.close_and_exit()
-            server.config.recovery_staging_path = recovery_staging_path
-        # If the backup is compressed but there is no recovery_staging_path
-        # then this is an error - the user *must* tell barman where recovery
-        # data can be staged.
-        if server.config.recovery_staging_path is None:
-            output.error(
-                "Cannot restore from backup '%s' of server '%s': "
-                "backup is compressed with %s compression but no recovery "
-                "staging path is provided. Either set recovery_staging_path "
-                "in the Barman config or use the --recovery-staging-path "
-                "argument.",
-                args.backup_id,
-                server.config.name,
-                backup_info.compression,
-            )
-            output.close_and_exit()
+    # TODO: rethink the logic below after we remove the deprecated
+    # recovery_staging_path and local_staging_path options
 
-    # If the backup to be recovered is incremental then there are additional
-    # checks to be carried out
-    if backup_info.is_incremental:
-        # Set the local staging path from the cli if it is set
-        if args.local_staging_path is not None:
-            try:
-                local_staging_path = parse_staging_path(args.local_staging_path)
-            except ValueError as exc:
-                output.error("Cannot parse local staging path: %s", str(exc))
+    # If the user does not have a staging path set, check if maybe they are using the
+    # old staging options (recovery_staging_path or local_staging_path), which are
+    # deprecated, but still supported. The whole block below handles that
+    if (
+        any(
+            [
+                backup_info.is_incremental,
+                backup_info.compression,
+                backup_info.encryption,
+            ]
+        )
+        and not server.config.staging_path
+    ):
+        if (
+            not server.config.recovery_staging_path
+            and not server.config.local_staging_path
+        ):
+            # As we can't put the default value in the config module until the
+            # deprecated options get dropped, we set the default here if they are not
+            # being used.
+            server.config.staging_path = "/tmp"
+        else:
+            # If the backup is encrypted and staging_path is not set, the equivalent old
+            # option local_staging_path should be
+            if backup_info.encryption and not server.config.local_staging_path:
+                output.error(
+                    "Cannot restore from backup '%s' of server '%s': "
+                    "backup is encrypted with '%s' and it will be decrypted in the "
+                    "barman host but no staging path and location was provided. "
+                    "Please set staging_path and staging_location in the server "
+                    "configuration file, or use their equivalent CLI options "
+                    "--staging-path and --staging-location",
+                    args.backup_id,
+                    server.config.name,
+                    backup_info.encryption,
+                )
                 output.close_and_exit()
-            server.config.local_staging_path = local_staging_path
-        # If the backup is incremental but there is no local_staging_path
-        # then this is an error - the user *must* tell barman where recovery
-        # data can be staged.
-        if server.config.local_staging_path is None:
-            output.error(
-                "Cannot restore from backup '%s' of server '%s': "
-                "backup will be combined with pg_combinebackup in the "
-                "barman host but no local staging path is provided. "
-                "Either set local_staging_path in the Barman config "
-                "or use the --local-staging-path argument.",
-                args.backup_id,
-                server.config.name,
+            # If the backup is compressed and staging_path is not set, the equivalent old
+            # option recovery_staging_path should be
+            if backup_info.compression and not server.config.recovery_staging_path:
+                output.error(
+                    "Cannot restore from backup '%s' of server '%s': "
+                    "backup is compressed with %s compression and it will be "
+                    "decompressed in a staging area but no staging "
+                    "path was provided. Please set staging_path and staging_location "
+                    "in the server configuration file, or use their equivalent CLI "
+                    "options --staging-path and --staging-location",
+                    args.backup_id,
+                    server.config.name,
+                    backup_info.compression,
+                )
+                output.close_and_exit()
+            # If the backup is incremental and staging_path is not set, the equivalent old
+            # option local_staging_path should be
+            if backup_info.is_incremental and not server.config.local_staging_path:
+                output.error(
+                    "Cannot restore from backup '%s' of server '%s': "
+                    "backup has to be combined with pg_combinebackup in a "
+                    "staging area but no staging path and location was provided. "
+                    "Please set staging_path and staging_location in the server "
+                    "configuration file, or use their equivalent CLI options "
+                    "--staging-path and --staging-location",
+                    args.backup_id,
+                    server.config.name,
+                )
+                output.close_and_exit()
+            # If we got here then the user is using one the old staging options
+            # which are deprecated, but supported, so issue a warning about it
+            output.warning(
+                "recovery_staging_path and local_staging_path, and their equivalent CLI "
+                "options --recovery-staging-path and --local-staging-path, are "
+                "deprecated and will be removed in a future release. "
+                "Please use staging_path and staging_location, and their equivalent CLI "
+                "options --staging-path and --staging-location, instead.",
             )
-            output.close_and_exit()
 
     # decode the tablespace relocation rules
     tablespaces = {}
@@ -1294,6 +1427,14 @@ def switch_wal(args):
         argument(
             "--nagios", help="Nagios plugin compatible output", action="store_true"
         ),
+        argument(
+            "--check-timeout",
+            help="Specifies the time limit, in seconds, for the check operation. "
+            "Set to zero to disable the timeout.",
+            metavar="TIMEOUT",
+            default=None,
+            type=check_non_negative,
+        ),
     ]
 )
 def check(args):
@@ -1309,6 +1450,8 @@ def check(args):
     for name in sorted(servers):
         server = servers[name]
 
+        if args.check_timeout is not None:
+            server.config.check_timeout = args.check_timeout
         # Validate the returned server
         if not manage_server_command(
             server,
@@ -1491,6 +1634,12 @@ def show_backup(args):
                        backup and the following one (if any) or the end of the log) and
                        full (same as data + wal). Defaults to %(default)s""",
         ),
+        argument(
+            "--list-empty-directories",
+            help="Also includes empty directories in the list.",
+            action="store_true",
+            default=False,
+        ),
     ]
 )
 def list_files(args):
@@ -1502,7 +1651,9 @@ def list_files(args):
     # Retrieves the backup
     backup_info = parse_backup_id(server, args)
     try:
-        for line in backup_info.get_list_of_files(args.target):
+        for line in backup_info.get_directory_entries(
+            args.target, empty_dirs=args.list_empty_directories
+        ):
             output.info(line, log=False)
     except BadXlogSegmentName as e:
         output.error(
@@ -1731,6 +1882,11 @@ def archive_wal(args):
             help="create the replication slot, if it does not exist",
             action="store_true",
         ),
+        argument(
+            "--if-not-exists",
+            help="Do not error out when --create-slot is specified and a slot with the specified name already exists.",
+            action="store_true",
+        ),
         argument(
             "--drop-slot",
             help="drop the replication slot, if it exists",
@@ -1762,7 +1918,7 @@ def receive_wal(args):
         server.kill("receive-wal")
     elif args.create_slot:
         with closing(server):
-            server.create_physical_repslot()
+            server.create_physical_repslot(ignore_duplicate=args.if_not_exists)
     elif args.drop_slot:
         with closing(server):
             server.drop_repslot()
