@@ -886,7 +886,7 @@ class ConsoleOutputWriter(object):
         # This is WAL stuff...
         wal_num = backup_info.get("wal_num")
         if wal_num:
-            output_fun(nested_row.format("WAL number", wal_num))
+            output_fun(nested_row.format("Number of WALs", wal_num))
 
         wal_compression_ratio = backup_info.get("wal_compression_ratio", 0)
         # Output WAL compression ratio for basebackup WAL files
@@ -963,7 +963,7 @@ class ConsoleOutputWriter(object):
         ):
             output_fun(header_row.format("WAL information"))
             output_fun(
-                nested_row.format("No of files", backup_info["wal_until_next_num"])
+                nested_row.format("Number of files", backup_info["wal_until_next_num"])
             )
             output_fun(
                 nested_row.format(
