@@ -37,7 +37,7 @@ def default_session_fixture(request):
     connect_mock.side_effect = psycopg2.DatabaseError
 
     def unpatch():
-        connect_patch.__exit__([None])
+        connect_patch.__exit__(None, None)
         logging.info("Unpatching barman.postgres.psycopg2.connect")
 
     request.addfinalizer(unpatch)
