@@ -21,7 +21,7 @@ import pytest
 from testing_helpers import build_test_backup_info
 
 from barman.clients import cloud_restore
-from barman.clients.cloud_cli import OperationErrorExit
+from barman.clients.cloud_cli import CLIErrorExit, OperationErrorExit
 from barman.clients.cloud_restore import (
     CloudBackupDownloaderObjectStore,
     CloudBackupDownloaderSnapshot,
@@ -341,7 +341,7 @@ class TestCloudRestore(object):
     @mock.patch("barman.clients.cloud_restore.get_backup_id_from_target_time")
     @mock.patch("barman.clients.cloud_restore.CloudBackupCatalog")
     @mock.patch("barman.clients.cloud_restore.get_cloud_interface")
-    def test_restore_without_backup_id(
+    def test_restore_with_backup_id_auto(
         self,
         _mock_cloud_interface_factory,
         mock_catalog,
@@ -355,10 +355,9 @@ class TestCloudRestore(object):
         expected_backup_id,
     ):
         """
-        Verify that restoring a backup without a backup_id will retrieve the
+        Verify that restoring a backup when backup_id is set to "auto" will retrieve the
         correct backup_id depending on the recovery target used. This tests the
-        `else` statement in the main cloud_restore function when backup_id is
-        None.
+        `else` statement in the main cloud_restore function.
         """
 
         backups = {
@@ -414,7 +413,7 @@ class TestCloudRestore(object):
             [
                 "cloud_storage_url",
                 "test_server",
-                None,
+                "auto",
                 recovery_dir,
             ]
             + target_args
@@ -448,7 +447,7 @@ class TestCloudRestore(object):
     @mock.patch("barman.clients.cloud_restore.get_last_backup_id")
     @mock.patch("barman.clients.cloud_restore.CloudBackupCatalog")
     @mock.patch("barman.clients.cloud_restore.get_cloud_interface")
-    def test_restore_without_backup_id_with_target_tli(
+    def test_restore_with_backup_id_auto_with_target_tli(
         self,
         _mock_cloud_interface_factory,
         mock_catalog,
@@ -461,10 +460,9 @@ class TestCloudRestore(object):
         parsed_tli,
     ):
         """
-        Verify that restoring a backup without a backup_id will retrieve the
+        Verify that restoring a backup when backup_id is set to "auto" will retrieve the
         correct backup_id depending on the recovery target used. This tests the
-        `else` statement in the main cloud_restore function when backup_id is
-        None.
+        `else` statement in the main cloud_restore function.
         """
 
         backups = {
@@ -518,7 +516,7 @@ class TestCloudRestore(object):
             [
                 "cloud_storage_url",
                 "test_server",
-                None,
+                "auto",
                 recovery_dir,
             ]
             + target_args
@@ -537,7 +535,7 @@ class TestCloudRestore(object):
     @mock.patch("barman.clients.cloud_restore.get_backup_id_from_target_time")
     @mock.patch("barman.clients.cloud_restore.CloudBackupCatalog")
     @mock.patch("barman.clients.cloud_restore.get_cloud_interface")
-    def test_restore_without_backup_id_and_no_backup_id_found_from_targets(
+    def test_restore_with_backup_id_auto_and_no_backup_id_found_from_targets(
         self,
         _mock_cloud_interface_factory,
         _mock_catalog,
@@ -545,7 +543,7 @@ class TestCloudRestore(object):
         logger,
     ):
         """
-        Verify that restoring a backup without a backup_id passing multipl
+        Verify that restoring a backup when backup_id is set to "auto" passing multiple
         recovery targets will fall into an error with a specific message.
         """
 
@@ -563,7 +561,7 @@ class TestCloudRestore(object):
                 [
                     "cloud_storage_url",
                     "test_server",
-                    None,
+                    "auto",
                     recovery_dir,
                 ]
                 + target_args
@@ -572,6 +570,46 @@ class TestCloudRestore(object):
         error_msg = "Cannot find any candidate backup for recovery."
         logger.assert_called_once_with(error_msg)
 
+    @mock.patch("barman.clients.cloud_restore.get_backup_id_from_target_time")
+    @mock.patch("barman.clients.cloud_restore.CloudBackupCatalog")
+    @mock.patch("barman.clients.cloud_restore.get_cloud_interface")
+    def test_restore_mutually_exclusive_targets(
+        self,
+        _mock_cloud_interface_factory,
+        _mock_catalog,
+        mock_get_bkp_id_from_tgt_time,
+        capsys,
+    ):
+        """
+        Verify that restoring a backup with ``target_time`` and ``target_lsn`` recovery
+        targets will fall into an error with a specific message.
+        """
+
+        recovery_dir = "/path/to/restore_dir"
+
+        target_args = [
+            "--target-time",
+            "2025-01-07 15:15:00",
+            "--target-lsn",
+            "3/5E000000",
+        ]
+        mock_get_bkp_id_from_tgt_time.return_value = None
+        with pytest.raises(CLIErrorExit, match="3"):
+            cloud_restore.main(
+                [
+                    "cloud_storage_url",
+                    "test_server",
+                    None,
+                    recovery_dir,
+                ]
+                + target_args
+            )
+        captured = capsys.readouterr()
+        assert (
+            "argument --target-lsn: not allowed with argument --target-time"
+            in captured.err
+        )
+
 
 class TestCloudBackupDownloader(object):
     """Superclass containing common fixtures for CloudBackupDownloader tests."""
