@@ -38,6 +38,7 @@ from barman.cli import (
     OrderedHelpFormatter,
     argument,
     backup,
+    check,
     check_target_action,
     check_wal_archive,
     command,
@@ -48,6 +49,7 @@ from barman.cli import (
     get_server,
     get_server_list,
     keep,
+    list_files,
     list_processes,
     manage_model_command,
     manage_server_command,
@@ -58,7 +60,7 @@ from barman.cli import (
     show_servers,
     terminate_process,
 )
-from barman.exceptions import WalArchiveContentError
+from barman.exceptions import BadXlogSegmentName, WalArchiveContentError
 from barman.infofile import BackupInfo
 from barman.server import Server
 
@@ -559,6 +561,8 @@ class TestCli(object):
         args.target_lsn = None
         args.recovery_staging_path = None
         args.local_staging_path = None
+        args.staging_path = None
+        args.staging_location = None
         return args
 
     @patch("barman.cli.parse_backup_id")
@@ -757,282 +761,196 @@ class TestCli(object):
         _out, err = capsys.readouterr()
         assert "" == err
 
-    @pytest.mark.parametrize(
-        (
-            "backup_is_compressed",
-            "recovery_staging_path_arg",
-            "recovery_staging_path_config",
-            "expected_recovery_staging_path",
-            "should_error",
-            "error_substring",
-        ),
-        [
-            # If a backup is not compressed then recovery_staging_path is ignored
-            (False, None, None, None, False, None),
-            # If a backup is compressed and no recovery_staging_path is provided
-            # we expect an error
-            (
-                True,
-                None,
-                None,
-                None,
-                True,
-                "backup is compressed with gzip compression but no recovery staging "
-                "path is provided.",
-            ),
-            # If a backup is compressed and an argument is provided then it should
-            # be set in the config
-            (True, "/from/arg", None, "/from/arg", False, None),
-            # If a backup is compressed and a bad argument is provided then it should
-            # error
-            (
-                True,
-                "from/arg",
-                None,
-                None,
-                True,
-                "Cannot parse recovery staging path: Invalid value : 'from/arg' (must "
-                "be an absolute path)",
-            ),
-            # If a backup is compressed and a config value is set then it should
-            # be set in the config
-            (True, None, "/from/conf", "/from/conf", False, None),
-            # If a backup is compressed and both arg and config are set then arg
-            # takes precedence
-            (True, "/from/arg", "/from/conf", "/from/arg", False, None),
-        ],
-    )
-    @patch("barman.cli.parse_backup_id")
     @patch("barman.cli.get_server")
-    def test_restore_recovery_staging_path(
-        self,
-        get_server_mock,
-        parse_backup_id_mock,
-        mock_backup_info,
-        mock_restore_args,
-        backup_is_compressed,
-        recovery_staging_path_arg,
-        recovery_staging_path_config,
-        expected_recovery_staging_path,
-        should_error,
-        error_substring,
-        monkeypatch,
-        capsys,
+    @patch("barman.cli.parse_staging_path")
+    def test_restore_staging_path(
+        self, mock_parser, mock_get_server, mock_restore_args, monkeypatch, capsys
     ):
-        # GIVEN a backup
-        parse_backup_id_mock.return_value = mock_backup_info
-        # AND the backup is not incremental
-        mock_backup_info.is_incremental = False
-        # AND the backup is not encrypted
-        mock_backup_info.encryption = None
-        # AND the backup has the specified compression
-        mock_backup_info.compression = backup_is_compressed and "gzip" or None
-        # AND a configuration with the specified recovery_staging_path
-        config = build_config_from_dicts(
-            global_conf={"recovery_staging_path": recovery_staging_path_config},
-        )
+        """
+        Test the restore CLI function with a staging path.
+        """
+        # GIVEN a server with a configuration
+        config = build_config_from_dicts()
         server = config.get_server("main")
-        get_server_mock.return_value.config = server
+        mock_get_server.return_value.config = server
         monkeypatch.setattr(
             barman,
             "__config__",
             (config,),
         )
-        # WHEN recover is called with the specified --recovery-staging-path
-        mock_restore_args.recovery_staging_path = recovery_staging_path_arg
 
-        # WITH a barman recover command
+        mock_restore_args.staging_path = "/some/staging/path"
+
+        # Case 1: a parsing error occurs
+        mock_parser.side_effect = ValueError("Parse error")
         with pytest.raises(SystemExit):
             restore(mock_restore_args)
-
-        # THEN if we expected an error the error was observed
+        # THEN an error is raised and logged
         _, err = capsys.readouterr()
-        errors = [msg for msg in err.split("\n") if msg.startswith("ERROR: ")]
-        if should_error:
-            assert len(err) > 0
-            assert any([error_substring in msg for msg in errors])
-        else:
-            assert len(errors) == 0
-            # AND if we expected success, the server config recovery staging
-            # path matches expectations
-            assert server.recovery_staging_path == expected_recovery_staging_path
+        assert "ERROR: Cannot parse staging path: Parse error" in err
+
+        # Case 2: a valid staging path is provided
+        mock_parser.side_effect = None
+        mock_parser.return_value = "/some/staging/path"
+        with pytest.raises(SystemExit):
+            restore(mock_restore_args)
+        # THEN the server's staging path is set correctly
+        assert server.staging_path == "/some/staging/path"
 
-    @pytest.mark.parametrize(
-        (
-            "backup_is_incremental_or_encrypted",
-            "local_staging_path_arg",
-            "local_staging_path_config",
-            "expected_local_staging_path",
-            "should_error",
-            "error_substring",
-        ),
-        [
-            # If a backup is not incremental or encrypted then local_staging_path is
-            # ignored
-            (False, None, None, None, False, None),
-            # If a backup is incremental or encrypted and an argument is provided then
-            # it should be set in the config
-            (True, "/from/arg", None, "/from/arg", False, None),
-            # If a backup is incremental or encrypted and a bad argument is provided
-            # then it should error
-            (
-                True,
-                "from/arg",
-                None,
-                None,
-                True,
-                "Cannot parse local staging path: Invalid value : 'from/arg' (must "
-                "be an absolute path)",
-            ),
-            # If a backup is incremental or encrypted and a config value is set then it
-            # should be set in the config
-            (True, None, "/from/conf", "/from/conf", False, None),
-            # If a backup is incremental or encrypted and both arg and config are set
-            # then arg takes precedence
-            (True, "/from/arg", "/from/conf", "/from/arg", False, None),
-        ],
-    )
-    @patch("barman.cli.parse_backup_id")
     @patch("barman.cli.get_server")
-    def test_restore_local_staging_path_similar_code_path_enc_or_inc(
+    def test_restore_staging_location(
         self,
-        get_server_mock,
-        parse_backup_id_mock,
-        mock_backup_info,
+        mock_get_server,
         mock_restore_args,
-        backup_is_incremental_or_encrypted,
-        local_staging_path_arg,
-        local_staging_path_config,
-        expected_local_staging_path,
-        should_error,
-        error_substring,
         monkeypatch,
         capsys,
     ):
         """
-        Test the behavior of the restore function when using a local staging path.
-        This test verifies that the restore function correctly handles the
-        --local-staging-path argument and the local_staging_path configuration
-        option. It ensures that errors are raised when expected and that the
-        server's local staging path is set correctly on success.
-        For incremental or encrypted backups, this unit test has the same outcome
-        independently. There is a specific test that will be responsible for the
-        specific case - `test_restore_local_staging_path_specific_code_path_enc_or_inc`.
+        Test the restore CLI function with a staging location.
         """
-        # GIVEN a backup
-        parse_backup_id_mock.return_value = mock_backup_info
-        # AND the backup is incremental, so it is not encrypted.
-        mock_backup_info.is_incremental = backup_is_incremental_or_encrypted
-        mock_backup_info.encryption = backup_is_incremental_or_encrypted
-        # AND a configuration with the specified local_staging_path
-        config = build_config_from_dicts(
-            global_conf={"local_staging_path": local_staging_path_config},
-        )
+        # GIVEN a server with a configuration
+        config = build_config_from_dicts()
         server = config.get_server("main")
-        get_server_mock.return_value.config = server
+        mock_get_server.return_value.config = server
         monkeypatch.setattr(
             barman,
             "__config__",
             (config,),
         )
-        # WHEN recover is called with the specified --local-staging-path
-        mock_restore_args.local_staging_path = local_staging_path_arg
 
-        # WITH a barman recover command
+        # Case 1: When remote-ssh-command is not set
+        mock_restore_args.staging_location = "remote"
+        mock_restore_args.remote_ssh_command = None
         with pytest.raises(SystemExit):
             restore(mock_restore_args)
-
-        # THEN if we expected an error the error was observed
+        # THEN an error is raised and logged
         _, err = capsys.readouterr()
-        errors = [msg for msg in err.split("\n") if msg.startswith("ERROR: ")]
-        if should_error:
-            assert len(err) > 0
-            assert any([error_substring in msg for msg in errors])
-        else:
-            assert len(errors) == 0
-            # AND if we expected success, the server config recovery staging
-            # path matches expectations
-            assert server.local_staging_path == expected_local_staging_path
+        assert (
+            "ERROR: --staging-location as remote requires "
+            "--remote-ssh-command to be set"
+        ) in err
+
+        # Case 2: When remote-ssh-command is set
+        mock_restore_args.remote_ssh_command = "ssh postgres@pg"
+        with pytest.raises(SystemExit):
+            restore(mock_restore_args)
+        # THEN the server's staging path is set correctly
+        assert server.staging_location == mock_restore_args.staging_location
 
     @pytest.mark.parametrize(
         (
-            "backup_is_incremental",
-            "backup_encryption",
-            "local_staging_path_arg",
-            "local_staging_path_config",
-            "error_substring",
+            "staging_path, recovery_staging_path, local_staging_path, is_incremental, compression, encryption, should_fail, should_warn"
         ),
         [
-            # If a backup is incremental and not encrypted and no local_staging_path is
-            # provided we expect an error
+            # Case 1: compressed backup, no staging_path, no recovery_staging_path -> should not fail
+            (None, None, None, False, "gzip", None, False, False),
+            # Case 2: compressed backup, no staging_path, recovery_staging_path set -> should warn, not fail
             (
-                True,
                 None,
+                "/path/to/some/recovery/staging",
                 None,
-                None,
-                "backup will be combined with pg_combinebackup in the barman host but "
-                "no local staging path is provided.",
-            ),
-            # If a backup is not incremental and encrypted and no local_staging_path is
-            # provided we expect an error
-            (
                 False,
-                "gpg",
+                "gzip",
                 None,
-                None,
-                "backup is encrypted with 'gpg' and it will be decrypted in the",
+                False,
+                True,
             ),
+            # Case 3: incremental backup, no staging_path, no local_staging_path -> should not fail
+            (None, None, None, True, None, None, False, False),
+            # Case 4: incremental backup, no staging_path, local_staging_path set -> should warn, not fail
+            (None, None, "/path/to/some/local/staging", True, None, None, False, True),
+            # Case 5: encrypted backup, no staging_path, no local_staging_path -> should fail
+            (None, None, None, False, None, "gpg", False, False),
+            # Case 6: encrypted backup, no staging_path, local_staging_path set -> should warn, not fail
+            (
+                None,
+                None,
+                "/path/to/some/local/staging",
+                False,
+                None,
+                "gpg",
+                False,
+                True,
+            ),
+            # Case 7: compressed backup, staging_path set -> should not fail, no warning
+            ("some/staging/path", None, None, False, "gzip", None, False, False),
+            # Case 8: incremental backup, staging_path set -> should not fail, no warning
+            ("some/staging/path", None, None, True, None, None, False, False),
+            # Case 9: encrypted backup, staging_path set -> should not fail, no warning
+            ("some/staging/path", None, None, False, None, "gpg", False, False),
         ],
     )
     @patch("barman.cli.parse_backup_id")
     @patch("barman.cli.get_server")
-    def test_restore_local_staging_path_specific_code_path_enc_or_inc(
+    def test_restore_staging_path_supports_deprecated_options(
         self,
-        get_server_mock,
+        mock_get_server,
         parse_backup_id_mock,
-        mock_backup_info,
         mock_restore_args,
-        backup_is_incremental,
-        backup_encryption,
-        local_staging_path_arg,
-        local_staging_path_config,
-        error_substring,
+        mock_backup_info,
         monkeypatch,
         capsys,
+        staging_path,
+        recovery_staging_path,
+        local_staging_path,
+        is_incremental,
+        compression,
+        encryption,
+        should_fail,
+        should_warn,
     ):
         """
-        Test a specific behavior of the restore function when using a local staging
-        path with encrypted or incremental backups. It ensures that errors are raised
-        when expected.
+        Test the restore CLI function considers and supports the deprecated
+        staging options when validating.
         """
-        # GIVEN a backup
-        parse_backup_id_mock.return_value = mock_backup_info
-        # AND the backup is encrypted, so it is not incremental.
-        mock_backup_info.encryption = backup_encryption
-        mock_backup_info.is_incremental = backup_is_incremental
-        # AND a configuration with the specified local_staging_path
-        config = build_config_from_dicts(
-            global_conf={"local_staging_path": local_staging_path_config},
-        )
+        # GIVEN a server with a configuration
+        config = build_config_from_dicts()
         server = config.get_server("main")
-        get_server_mock.return_value.config = server
+        mock_get_server.return_value.config = server
         monkeypatch.setattr(
             barman,
             "__config__",
             (config,),
         )
-        # WHEN recover is called with the specified --local-staging-path
-        mock_restore_args.local_staging_path = local_staging_path_arg
 
-        # WITH a barman recover command
+        # Set the staging path options
+        mock_restore_args.staging_path = staging_path
+        mock_restore_args.recovery_staging_path = recovery_staging_path
+        mock_restore_args.local_staging_path = local_staging_path
+
+        # Set relevant backup_info attributes
+        parse_backup_id_mock.return_value = mock_backup_info
+        mock_backup_info.is_incremental = is_incremental
+        mock_backup_info.compression = compression
+        mock_backup_info.encryption = encryption
+        mock_backup_info.server = server
+
+        # IF it should fail THEN an error is raised and logged
+        if should_fail:
+            with pytest.raises(SystemExit):
+                restore(mock_restore_args)
+            _, err = capsys.readouterr()
+            assert "ERROR: Cannot restore from backup" in err
+            return
+
+        # ELSE it runs successfully
         with pytest.raises(SystemExit):
             restore(mock_restore_args)
 
-        # THEN if we expected an error the error was observed
-        _, err = capsys.readouterr()
-        errors = [msg for msg in err.split("\n") if msg.startswith("ERROR: ")]
-        assert len(err) > 0
-        assert any([error_substring in msg for msg in errors])
+        # AND IF it should warn THEN a warning is logged about deprecated options
+        if should_warn:
+            _, err = capsys.readouterr()
+            assert (
+                "WARNING: recovery_staging_path and local_staging_path, and their equivalent CLI "
+                "options --recovery-staging-path and --local-staging-path, are "
+                "deprecated and will be removed in a future release. "
+                "Please use staging_path and staging_location, and their equivalent CLI "
+                "options --staging-path and --staging-location, instead." in err
+            )
+        # ELSE no warning is logged
+        else:
+            _, err = capsys.readouterr()
+            assert "WARNING" not in err
 
     @pytest.mark.parametrize(
         ("status", "should_error"),
@@ -1847,6 +1765,137 @@ class TestCli(object):
         dummy_server.kill.assert_called_once_with(args.task)
         mock_output.close_and_exit.assert_called_once()
 
+    @pytest.mark.parametrize(
+        "arg_timeout, config_timeout, expected_timeout",
+        [(None, None, 30), (None, 300, 300), (600, 300, 600)],
+    )
+    @patch("barman.output.close_and_exit")
+    @patch("barman.cli.get_server_list")
+    def test_backup_check_timeout_cli_arg_overrides_config_option(
+        self,
+        mock_get_server_list,
+        mock_close_and_exit,
+        arg_timeout,
+        config_timeout,
+        expected_timeout,
+    ):
+        """
+        Verify the check_timeout from args overrides check_timeout from configuration
+        option when running a backup command.
+        """
+        config = {"main_conf": {"check_timeout": config_timeout}}
+        server = build_real_server(**config if config_timeout is not None else {})
+
+        mock_server_list = {"main": server}
+        mock_get_server_list.return_value = mock_server_list
+        mock_close_and_exit.return_value = None
+
+        args = Mock()
+        args.sever_name = "main"
+        args.check_timeout = arg_timeout
+        args.backup_id = "test_backup"
+        backup(args)
+        assert server.config.check_timeout == expected_timeout
+
+    @pytest.mark.parametrize(
+        "arg_timeout, config_timeout, expected_timeout",
+        [(None, None, 30), (None, 300, 300), (600, 300, 600)],
+    )
+    @patch("barman.cli.output")
+    @patch("barman.cli.get_server_list")
+    def test_check_check_timeout_cli_arg_overrides_config_option(
+        self,
+        mock_get_server_list,
+        mock_output,
+        arg_timeout,
+        config_timeout,
+        expected_timeout,
+    ):
+        """
+        Verify the check_timeout from args overrides check_timeout from configuration
+        option when running a backup command.
+        """
+        config = {"main_conf": {"check_timeout": config_timeout}}
+        server = build_mocked_server(**config if config_timeout is not None else {})
+        server.config.active = True
+        server.config.active = False
+        mock_server_list = {"main": server}
+        mock_get_server_list.return_value = mock_server_list
+
+        args = Mock()
+        args.sever_name = "main"
+        args.check_timeout = arg_timeout
+        args.backup_id = "test_backup"
+        args.nagios = False  # This has to be assigned to `False`, otherwise this is a
+        # leaky tests. the `output.set_output_writer` uses a global
+        # variable that will impact other tests.
+        check(args)
+        assert server.config.check_timeout == expected_timeout
+        server.check.assert_called_once_with()
+        mock_output.init.assert_called_once_with("check", "main", False, False)
+        mock_output.close_and_exit.assert_called_once_with()
+
+    @patch("barman.cli.parse_backup_id")
+    @patch("barman.cli.get_server")
+    @patch("barman.cli.output")
+    def test_list_files(
+        self,
+        mock_output,
+        mock_get_server,
+        mock_parse_backup,
+    ):
+        """
+        Test that `list_files` yields files under the backup directory for
+        each target and empty dirs when requested.
+        """
+        args = Mock()
+        args.sever_name = "test_server"
+        args.backup_id = "test_backup_id"
+        args.target = "data"
+        args.list_empty_directories = False
+        dummy_server = Mock()
+        dummy_server.config.name = "test_server"
+        mock_get_server.return_value = dummy_server
+
+        mock_parse_backup.return_value.backup_id = "test_backup_id"
+
+        mock_parse_backup.return_value.get_directory_entries.return_value = [
+            "non-empty_dir/file.txt"
+        ]
+
+        list_files(args)
+        mock_output.info.assert_called_once_with("non-empty_dir/file.txt", log=False)
+        mock_parse_backup.return_value.get_directory_entries.assert_called_once_with(
+            "data", empty_dirs=False
+        )
+
+    @patch("barman.cli.parse_backup_id")
+    @patch("barman.cli.get_server")
+    def test_list_files_bad_xlog_segment_name(
+        self, mock_get_server, mock_parse_backup_id, capsys
+    ):
+        """
+        Test list_files handles BadXlogSegmentName exception.
+        """
+        mock_server = Mock()
+        mock_server.config.name = "main"
+        mock_backup_info = Mock()
+        mock_backup_info.get_directory_entries.side_effect = BadXlogSegmentName(
+            "badseg"
+        )
+        mock_parse_backup_id.return_value = mock_backup_info
+        mock_get_server.return_value = mock_server
+        args = Mock()
+        args.server_name = "main"
+        args.backup_id = "20190101T000000"
+        args.target = "standalone"
+        args.list_empty_directories = False
+        with pytest.raises(SystemExit):
+            list_files(args)
+        out, err = capsys.readouterr()
+        assert "invalid xlog segment name" in err
+        assert 'Please run "barman rebuild-xlogdb main"' in err
+
 
 class TestKeepCli(object):
     @pytest.fixture
