@@ -36,6 +36,7 @@ from barman.config import (
     CsvOption,
     ModelConfig,
     RecoveryOptions,
+    ServerConfig,
     parse_backup_compression,
     parse_backup_compression_format,
     parse_backup_compression_location,
@@ -1416,6 +1417,8 @@ class TestModelConfig:
             "snapshot_provider": None,
             "snapshot_zone": None,
             "ssh_command": None,
+            "staging_path": None,
+            "staging_location": None,
             "streaming_archiver": None,
             "streaming_archiver_batch_size": None,
             "streaming_archiver_name": None,
@@ -1526,6 +1529,8 @@ class TestModelConfig:
             "snapshot_provider": {"source": "SOME_SOURCE", "value": None},
             "snapshot_zone": {"source": "SOME_SOURCE", "value": None},
             "ssh_command": {"source": "SOME_SOURCE", "value": None},
+            "staging_path": {"source": "SOME_SOURCE", "value": None},
+            "staging_location": {"source": "SOME_SOURCE", "value": None},
             "streaming_archiver": {"source": "SOME_SOURCE", "value": None},
             "streaming_archiver_batch_size": {"source": "SOME_SOURCE", "value": None},
             "streaming_archiver_name": {"source": "SOME_SOURCE", "value": None},
@@ -1994,6 +1999,17 @@ class TestBaseConfig:
 
 
 class TestConfigChangesProcessor:
+    @pytest.fixture(autouse=True)
+    def _allow_test_keys(self, monkeypatch):
+        """
+        In these unit tests we treat 'key1','key2','key3','key4'
+        as valid server options and 'key5','key6' as valid model options.
+        """
+        monkeypatch.setattr(ServerConfig, "KEYS", {"key1", "key2", "key3", "key4"})
+        monkeypatch.setattr(ServerConfig, "PARSERS", {})  # no special parsing
+        monkeypatch.setattr(ModelConfig, "KEYS", {"key5", "key6", "model", "cluster"})
+        monkeypatch.setattr(ModelConfig, "PARSERS", {})
+
     def test_receive_config_changes_existing_config(self, tmpdir):
         # test it is okay when updating existing servers and models config
         config = Mock()
@@ -2002,9 +2018,11 @@ class TestConfigChangesProcessor:
         queue_file.ensure(file=True)
         config.barman_home = tmpdir.strpath
         config.config_changes_queue = queue_file.strpath
+        # always override default
         config._config.get_config_source.return_value = "default"
-        config.get_server.side_effect = [Mock()] * 2 + [None]
-        config.get_model.side_effect = [None] * 2 + [Mock()]
+        # pretend the first two calls to get_server succeed, third yields None
+        config.get_server.side_effect = [Mock(), Mock(), None]
+        config.get_model.side_effect = [None, None, Mock()]
         processor = ConfigChangesProcessor(config)
 
         changes = [
@@ -2016,12 +2034,12 @@ class TestConfigChangesProcessor:
             },
             {
                 "server_name": "web",
-                "key3": "value3",
-                "key4": "value4",
+                "key1": "value3",
+                "key2": "value4",
                 "scope": "server",
             },
             {
-                "model_name": "my-model",
+                "model_name": "mymodel",
                 "key5": "value5",
                 "key6": "value6",
                 "scope": "model",
@@ -2030,146 +2048,113 @@ class TestConfigChangesProcessor:
 
         processor.receive_config_changes(changes)
 
-        with ConfigChangesQueue(config.config_changes_queue) as chgs_queue:
-            assert len(chgs_queue.queue) == 3
-            assert isinstance(chgs_queue.queue[0], ConfigChangeSet)
-            assert isinstance(chgs_queue.queue[1], ConfigChangeSet)
+        with ConfigChangesQueue(config.config_changes_queue) as q:
+            # we should have enqueued all three valid sections
+            assert len(q.queue) == 3
+            assert [cs.section for cs in q.queue] == ["main", "web", "mymodel"]
 
-            assert len(chgs_queue.queue[0].changes_set) == 2
-            assert isinstance(chgs_queue.queue[0].changes_set[0], ConfigChange)
-            assert isinstance(chgs_queue.queue[0].changes_set[1], ConfigChange)
-            assert chgs_queue.queue[0].section == "main"
-            assert chgs_queue.queue[0].changes_set[0].key == "key1"
-            assert chgs_queue.queue[0].changes_set[0].value == "value1"
-            assert chgs_queue.queue[0].changes_set[1].key == "key2"
-            assert chgs_queue.queue[0].changes_set[1].value == "value2"
+            # expected key/value pairs per section
+            expected = {
+                "main": [("key1", "value1"), ("key2", "value2")],
+                "web": [("key1", "value3"), ("key2", "value4")],
+                "mymodel": [("key5", "value5"), ("key6", "value6")],
+            }
+            for cs in q.queue:
+                # each has exactly 2 ConfigChange entries
+                assert len(cs.changes_set) == 2
+                for change in cs.changes_set:
+                    # correct key/value
+                    assert (change.key, change.value) in expected[cs.section]
+                    # config_file should be the auto.conf under barman_home
+                    assert change.config_file.endswith("/.barman.auto.conf")
 
-            assert len(chgs_queue.queue[1].changes_set) == 2
-            assert isinstance(chgs_queue.queue[1].changes_set[0], ConfigChange)
-            assert isinstance(chgs_queue.queue[1].changes_set[1], ConfigChange)
-            assert chgs_queue.queue[1].section == "web"
-            assert chgs_queue.queue[1].changes_set[0].key == "key3"
-            assert chgs_queue.queue[1].changes_set[0].value == "value3"
-            assert chgs_queue.queue[1].changes_set[1].key == "key4"
-            assert chgs_queue.queue[1].changes_set[1].value == "value4"
-
-            assert len(chgs_queue.queue[2].changes_set) == 2
-            assert isinstance(chgs_queue.queue[2].changes_set[0], ConfigChange)
-            assert isinstance(chgs_queue.queue[2].changes_set[1], ConfigChange)
-            assert chgs_queue.queue[2].section == "my-model"
-            assert chgs_queue.queue[2].changes_set[0].key == "key5"
-            assert chgs_queue.queue[2].changes_set[0].value == "value5"
-            assert chgs_queue.queue[2].changes_set[1].key == "key6"
-            assert chgs_queue.queue[2].changes_set[1].value == "value6"
-
-    @patch("barman.config.output.warning")
-    def test_receive_config_changes_warnings(self, mock_warning, tmpdir):
-        # test it throws the expected warnings when invalid requests are issued
-        # and that it ignores the changes instead of applying them
-        config = Mock()
+    @patch("barman.config.output.close_and_exit", side_effect=SystemExit)
+    @patch("barman.config.output.error")
+    def test_receive_config_changes_aborts_on_invalid(
+        self, mock_err, mock_exit, tmpdir
+    ):
+        cfg = Mock()
         queue_file = tmpdir.join("cfg_changes.queue")
         queue_file.write("[]")
         queue_file.ensure(file=True)
-        config.barman_home = tmpdir.strpath
-        config.config_changes_queue = queue_file.strpath
-        config._config.get_config_source.return_value = "default"
-        config.get_server.side_effect = [None, Mock(), None]
-        config.get_model.side_effect = [Mock(), None, None]
-        processor = ConfigChangesProcessor(config)
+        cfg.barman_home = tmpdir.strpath
+        cfg.config_changes_queue = queue_file.strpath
+        cfg.get_server.return_value = None
+        cfg.get_model.return_value = None
 
+        processor = ConfigChangesProcessor(cfg)
         changes = [
-            {"scope": "random"},  # invalid scope
-            {"scope": "server"},  # missing server_name
-            {"scope": "model"},  # missing model_name
-            {"scope": "server", "server_name": "my-model"},  # server is a model
-            {"scope": "model", "model_name": "my-server"},  # model is a server
-            {"scope": "model", "model_name": "my-model"},  # new model missing cluster
+            {"scope": "not-a-scope"},
+            {"server_name": "s1", "key1": "v1", "scope": "server"},
         ]
 
-        processor.receive_config_changes(changes)
+        with pytest.raises(SystemExit):
+            processor.receive_config_changes(changes)
 
-        with ConfigChangesQueue(config.config_changes_queue) as chgs_queue:
-            assert len(chgs_queue.queue) == 0
-
-        mock_warning.assert_has_calls(
-            [
-                call(
-                    "%r has been ignored because 'scope' is invalid: '%s'. It should be either 'server' or 'model'.",
-                    {"scope": "random"},
-                    "random",
-                ),
-                call(
-                    "%r has been ignored because 'server_name' is missing.",
-                    {"scope": "server"},
-                ),
-                call(
-                    "%r has been ignored because 'model_name' is missing.",
-                    {"scope": "model"},
-                ),
-                call(
-                    "%r has been ignored because '%s' is a model, not a server.",
-                    {"scope": "server", "server_name": "my-model"},
-                    "my-model",
-                ),
-                call(
-                    "%r has been ignored because '%s' is a server, not a model.",
-                    {"scope": "model", "model_name": "my-server"},
-                    "my-server",
-                ),
-                call(
-                    "%r has been ignored because it is a new model but 'cluster' is missing.",
-                    {"scope": "model", "model_name": "my-model"},
-                ),
-            ]
+        # expected message when aborting
+        mock_err.assert_called_with(
+            "Aborting config‐update: invalid section %r", {"scope": "not-a-scope"}
         )
+        # expected to call error both on exit and validation
+        assert mock_err.call_count == 2
+        mock_exit.assert_called_once()
+
+        # queue still empty
+        with ConfigChangesQueue(str(queue_file)) as q:
+            assert q.queue == []
+
+    @patch("barman.config.output.close_and_exit", side_effect=SystemExit)
+    @patch("barman.config.output.error")
+    def test_receive_config_changes_all_invalid(self, mock_err, mock_exit, tmpdir):
+        cfg = Mock()
+        queue_file = tmpdir.join("cfg_changes.queue")
+        queue_file.write("[]")
+        queue_file.ensure(file=True)
+        cfg.barman_home = tmpdir.strpath
+        cfg.config_changes_queue = queue_file.strpath
+
+        processor = ConfigChangesProcessor(cfg)
+        changes = [{"scope": "foo"}, {"scope": "bar"}]
+
+        with pytest.raises(SystemExit):
+            processor.receive_config_changes(changes)
+
+        # only the first invalid section should trigger error + exit
+        mock_err.assert_called_with(
+            "Aborting config‐update: invalid section %r", {"scope": "foo"}
+        )
+        assert mock_err.call_count == 2
+        mock_exit.assert_called_once()
+
+        with ConfigChangesQueue(str(queue_file)) as q:
+            assert q.queue == []
 
     @patch("barman.config.output.warning")
     def test_receive_config_changes_with_empty_or_malformed_queue_file(
         self, mock_warning, tmpdir
     ):
         # test it throws the expected warnings when invalid requests are issued
-        # and that it ignores the changes instead of applying them
+        # and that it aborts on invalid payload
         config = Mock()
         queue_file = tmpdir.join("cfg_changes.queue")
-        queue_file.ensure(file=True)
+        queue_file.ensure(file=True)  # empty file
         config.barman_home = tmpdir.strpath
         config.config_changes_queue = queue_file.strpath
         config._config.get_config_source.return_value = "default"
-        config.get_server.side_effect = [Mock()] * 2 + [None]
-        config.get_model.side_effect = [None] * 2 + [Mock()]
+        config.get_server.side_effect = [Mock(), None, None]
+        config.get_model.side_effect = [None, Mock(), None]
         processor = ConfigChangesProcessor(config)
 
-        changes = [
-            {
-                "server_name": "main",
-                "key1": "value1",
-                "key2": "value2",
-                "scope": "server",
-            }
-        ]
-
+        # single *valid* section → should enqueue
+        changes = [{"server_name": "main", "key1": "value1", "scope": "server"}]
         processor.receive_config_changes(changes)
 
-        with ConfigChangesQueue(config.config_changes_queue) as chgs_queue:
-            assert len(chgs_queue.queue) == 1
-            assert isinstance(chgs_queue.queue[0], ConfigChangeSet)
+        with ConfigChangesQueue(config.config_changes_queue) as q:
+            assert len(q.queue) == 1
 
-            assert len(chgs_queue.queue[0].changes_set) == 2
-            assert isinstance(chgs_queue.queue[0].changes_set[0], ConfigChange)
-            assert isinstance(chgs_queue.queue[0].changes_set[1], ConfigChange)
-            assert chgs_queue.queue[0].section == "main"
-            assert chgs_queue.queue[0].changes_set[0].key == "key1"
-            assert chgs_queue.queue[0].changes_set[0].value == "value1"
-            assert chgs_queue.queue[0].changes_set[1].key == "key2"
-            assert chgs_queue.queue[0].changes_set[1].value == "value2"
-
-        mock_warning.assert_has_calls(
-            [
-                call(
-                    "Malformed or empty configuration change queue: %s"
-                    % str(queue_file)
-                )
-            ]
+        # but because the queue-file was empty at start, we warned about that
+        mock_warning.assert_called_once_with(
+            "Malformed or empty configuration change queue: %s" % str(queue_file)
         )
 
     @patch("barman.config.output.info")
@@ -2226,7 +2211,6 @@ class TestConfigChangesProcessor:
         ]
 
         processor.receive_config_changes(changes)
-        # with patch("builtins.open", mock_open()) as mock_file:
         processor.process_conf_changes_queue()
 
         assert len(processor.applied_changes) == 2
@@ -2245,6 +2229,147 @@ class TestConfigChangesProcessor:
         )
 
 
+class TestValidateSection:
+    @pytest.fixture(autouse=True)
+    def _setup_keys_parsers(self, monkeypatch):
+        # For these tests treat 'k1','k2' as valid server options
+        monkeypatch.setattr(ServerConfig, "KEYS", {"k1", "k2"})
+        monkeypatch.setattr(ServerConfig, "PARSERS", {})
+        # And 'm1','cluster','model' as valid model options
+        monkeypatch.setattr(ModelConfig, "KEYS", {"m1", "cluster", "model"})
+        monkeypatch.setattr(ModelConfig, "PARSERS", {})
+
+    @pytest.fixture
+    def processor(self):
+        cfg = Mock()
+        cfg.get_server.return_value = None
+        cfg.get_model.return_value = None
+        return ConfigChangesProcessor(cfg)
+
+    @patch("barman.config.output.error")
+    def test_scope_invalid(self, mock_err, processor):
+        raw = {"scope": "bad"}
+        ok, name, opts = processor._validate_section(raw.copy())
+        assert not ok and name is None and opts is None
+        mock_err.assert_called_once_with(
+            "Invalid section %r: 'scope' is invalid: '%s'. It should be either 'server' or 'model'.",
+            raw,
+            "bad",
+        )
+
+    @patch("barman.config.output.error")
+    def test_missing_server_name(self, mock_err, processor):
+        raw = {"scope": "server", "foo": "bar"}
+        ok, name, opts = processor._validate_section(raw.copy())
+        assert not ok and name is None and opts is None
+        mock_err.assert_called_once_with(
+            "Invalid section %r: '%s' is missing.",
+            raw,
+            "server_name",
+        )
+
+    @patch("barman.config.output.error")
+    def test_missing_model_name(self, mock_err, processor):
+        raw = {"scope": "model", "foo": "bar"}
+        ok, name, opts = processor._validate_section(raw.copy())
+        assert not ok and name is None and opts is None
+        mock_err.assert_called_once_with(
+            "Invalid section %r: '%s' is missing.",
+            raw,
+            "model_name",
+        )
+
+    @patch("barman.config.output.error")
+    def test_server_conflicts_with_model(self, mock_err, processor):
+        processor.config.get_model.return_value = object()
+        raw = {"scope": "server", "server_name": "s1"}
+        ok, _, _ = processor._validate_section(raw.copy())
+        assert not ok
+        mock_err.assert_called_once_with(
+            "Invalid section %r: '%s' is a model, not a server.",
+            raw,
+            "s1",
+        )
+
+    @patch("barman.config.output.error")
+    def test_model_conflicts_with_server(self, mock_err, processor):
+        processor.config.get_server.return_value = object()
+        raw = {"scope": "model", "model_name": "m1", "cluster": "c1"}
+        ok, _, _ = processor._validate_section(raw.copy())
+        assert not ok
+        mock_err.assert_called_once_with(
+            "Invalid section %r: '%s' is a server, not a model.",
+            raw,
+            "m1",
+        )
+
+    @patch("barman.config.output.error")
+    def test_new_model_missing_cluster(self, mock_err, processor):
+        raw = {"scope": "model", "model_name": "m1"}
+        ok, _, _ = processor._validate_section(raw.copy())
+        assert not ok
+        mock_err.assert_called_once_with(
+            "Invalid section %r: new model but 'cluster' is missing.",
+            raw,
+        )
+
+    @patch("barman.config.output.error")
+    def test_new_model_with_cluster(self, mock_err, processor):
+        raw = {"scope": "model", "model_name": "m1", "cluster": "c1"}
+        ok, name, opts = processor._validate_section(raw.copy())
+        assert ok and name == "m1"
+        # use model=on plus existing cluster
+        assert opts == {"cluster": "c1", "model": "on"}
+        mock_err.assert_not_called()
+
+    @patch("barman.config.output.error")
+    def test_invalid_option_key(self, mock_err, processor):
+        raw = {"scope": "server", "server_name": "s", "bad": "x"}
+        ok, _, _ = processor._validate_section(raw.copy())
+        assert not ok
+        mock_err.assert_called_once_with(
+            "Invalid option '%s' for %s '%s'",
+            "bad",
+            "server",
+            "s",
+        )
+
+    @patch("barman.config.output.error")
+    def test_value_parsing_invalid(self, mock_err, processor, monkeypatch):
+        # parser for k1 raises
+        def fail(v):
+            raise ValueError("oops")
+
+        monkeypatch.setattr(ServerConfig, "PARSERS", {"k1": fail})
+        raw = {"scope": "server", "server_name": "s", "k1": "v1"}
+        ok, _, _ = processor._validate_section(raw.copy())
+        assert not ok
+        fmt, scope_arg, name_arg, key_arg, err = mock_err.call_args[0]
+        assert fmt == "Validation failed for %s '%s': key '%s' -> %s"
+        assert (scope_arg, name_arg, key_arg) == ("server", "s", "k1")
+        assert isinstance(err, ValueError)
+
+    @patch("barman.config.output.error")
+    def test_value_parsing_valid(self, mock_err, processor, monkeypatch):
+        # parser for k1 succeeds
+        def good(v):
+            return v
+
+        monkeypatch.setattr(ServerConfig, "PARSERS", {"k1": good})
+        raw = {"scope": "server", "server_name": "s", "k1": "v1"}
+        ok, name, opts = processor._validate_section(raw.copy())
+        assert ok and name == "s"
+        assert opts == {"k1": "v1"}
+        mock_err.assert_not_called()
+
+    def test_unknown_key_accepted(self, processor):
+        # k2 in KEYS but with no parser is accepted
+        raw = {"scope": "server", "server_name": "s", "k2": "v2"}
+        ok, name, opts = processor._validate_section(raw.copy())
+        assert ok and name == "s"
+        assert opts == {"k2": "v2"}
+
+
 class TestConfigChangeQueue:
     def test_config_changes_queue(self, tmpdir):
         config = Mock()
