@@ -17,6 +17,7 @@
 # along with Barman.  If not, see <http://www.gnu.org/licenses/>.
 
 import copy
+import io
 import json
 import os
 import warnings
@@ -33,6 +34,7 @@ from testing_helpers import (
     build_test_backup_info,
 )
 
+from barman import xlog
 from barman.cloud_providers.aws_s3 import AwsSnapshotMetadata, AwsSnapshotsInfo
 from barman.cloud_providers.azure_blob_storage import (
     AzureSnapshotMetadata,
@@ -47,7 +49,6 @@ from barman.infofile import (
     Field,
     FieldListFile,
     LocalBackupInfo,
-    SyntheticBackupInfo,
     VolatileBackupInfo,
     WalFileInfo,
     dump_backup_ids,
@@ -1508,77 +1509,98 @@ class TestLocalBackupInfo:
 
         assert not backup_info.is_full
 
-
-class TestSyntheticBackupInfo:
-    """
-    this class tests the methods of the SyntheticBackupInfo object
-    """
-
-    def test_init_synthetic_backup_info_with_backup_id(self):
+    @pytest.mark.parametrize("target", ["data", "standalone", "full", "wal"])
+    @patch("barman.infofile.LocalBackupInfo.get_required_wal_segments")
+    @patch("barman.infofile.LocalBackupInfo.get_basebackup_directory")
+    def test_get_directory_entries(
+        self,
+        mock_base_backup_dir,
+        mock_required_wal_segments,
+        target,
+        tmpdir,
+    ):
         """
-        Unit test for the __init__ method using backup_id.
-
-        Create mock server and a SyntheticBackupInfo object.
-
-        This unit tests checks:
-            * base_directory parameter
-            * backup_id parameter
-            * instance type
+        Test that `get_directory_entries` yields files under the backup directory for
+        each target and empty dirs when requested.
         """
-        server = build_mocked_server()
-        base_directory = "fake/path/"
-        backup_id = "fake_name"
-        obj = SyntheticBackupInfo(
-            server=server,
-            base_directory=base_directory,
-            backup_id=backup_id,
-            info_file=None,
+        backup_id = "test_backup"
+        server = build_mocked_server(main_conf={"backup_directory": tmpdir.strpath})
+        base_dir = tmpdir.mkdir("base")
+        wals_dir = tmpdir.mkdir("wals")
+        backup_dir = base_dir.mkdir(backup_id)
+        data_dir = backup_dir.mkdir("data")
+        # Create empty subdirectories
+        empty1 = data_dir.mkdir("empty1")
+        empty2 = data_dir.mkdir("empty2")
+        # Create a non-empty directory
+        non_empty = data_dir.mkdir("not_empty")
+        file_path = non_empty.join("file.txt")
+        file_path.write("content")
+        b_info = LocalBackupInfo(server, backup_id=backup_id)
+        mock_base_backup_dir.return_value = backup_dir
+        segment_name = "%08X%08X%08X" % (1, 1, 0)
+        mock_required_wal_segments.return_value = [segment_name]
+        server.get_wal_full_path.side_effect = lambda x: os.path.join(
+            wals_dir,
+            xlog.hash_dir(x),
+            x,
+        )
+        wal1_file_path = os.path.join(
+            wals_dir,
+            xlog.hash_dir(segment_name),
+            segment_name,
         )
-        assert obj.base_directory == base_directory
-        assert obj.backup_id == backup_id
-        assert isinstance(obj, SyntheticBackupInfo)
 
-    def test_init_synthetic_backup_info_with_info_file(self, tmpdir):
-        """
-        Unit test for the __init__ method using info_file.
+        server.get_wal_until_next_backup.return_value = [
+            WalFileInfo.from_xlogdb_line("000000000000000000000002 42 43 none none ")
+        ]
 
-        Create mock server and a SyntheticBackupInfo object.
+        wal2_file_path = os.path.join(
+            wals_dir,
+            "0000000000000000",
+            "000000000000000000000002",
+        )
 
-        This unit tests checks:
-            * base_directory parameter
-            * filename parameter
-            * instance type
-        """
+        # Collect all yielded
+        entries_list = list(b_info.get_directory_entries(target, empty_dirs=True))
+        # Should include all files and empty directories.
+        if target in ("full", "standalone", "data"):
+            assert str(file_path) in entries_list
+            assert str(empty1) in entries_list
+            assert str(empty2) in entries_list
+        # Should include only the WAL from `get_required_wal_segments`, not the one from
+        # `get_wal_until_next_backup`
+        if target == "standalone":
+            assert str(wal1_file_path) in entries_list
+            assert str(wal2_file_path) not in entries_list
+        # Should include only the WAL from `get_wal_until_next_backup`, not the one from
+        # `get_required_wal_segments`
+        if target in ("full", "wal"):
+            assert str(wal2_file_path) in entries_list
+            assert str(wal1_file_path) not in entries_list
+        # Should not include WALs
+        if target == "data":
+            assert str(wal2_file_path) not in entries_list
+            assert str(wal1_file_path) not in entries_list
+
+        # Collect only "files" yielded
+        entries_list = list(b_info.get_directory_entries(target))
+        # Should not include empty directories.
+        if target in ("full", "standalone", "data"):
+            assert str(file_path) in entries_list
+            assert str(empty1) not in entries_list
+            assert str(empty2) not in entries_list
+
+    def test_load_unknown_field(self, tmpdir, caplog):
         server = build_mocked_server()
-        base_directory = "fake/path/"
-        backup_id = "fake_name"
-        infofile = tmpdir.mkdir(backup_id).join("backup.info")
+        infofile = tmpdir.mkdir("backup_id").join("backup.info")
         infofile.write(BASE_BACKUP_INFO)
-        obj = SyntheticBackupInfo(
-            server=server,
-            base_directory=base_directory,
-            backup_id=None,
-            info_file=infofile.strpath,
-        )
-        assert obj.base_directory == base_directory
-        assert obj.filename == infofile.strpath
-        assert isinstance(obj, SyntheticBackupInfo)
+        infofile.write("dummy=15\n")
+        with pytest.raises(SystemExit):
+            dummy = LocalBackupInfo(server=server, info_file=infofile.strpath)
+            dummy.load(infofile.strpath)
 
-    def test_get_basebackup_directory(self):
-        """
-        Unit test for the get_basebackup_directory.
-
-        Create mock server and a SyntheticBackupInfo object.
-
-        This unit tests checks if the method returns the correct path based on
-        base_directory and backup_id.
-        """
-        server = build_mocked_server()
-        backup_info = SyntheticBackupInfo(
-            server=server, base_directory="/fake/path/", backup_id="fake_name"
-        )
-        directory = backup_info.get_basebackup_directory()
-        assert directory == "/fake/path/fake_name"
+        assert "Unsupported field 'dummy' found in backup metadata" in caplog.text
 
 
 class TestVolatileBackupInfo:
@@ -1628,10 +1650,10 @@ class TestVolatileBackupInfo:
         expected_directory = os.path.join(base_directory, backup_id)
         assert volatile_backup_info.get_basebackup_directory() == expected_directory
 
-    def test_volatile_backup_info_save_not_implemented(self):
+    def test_volatile_backup_info_save_to_file_not_allowed(self):
         """
         Test that the :meth:`save` method of :class:`VolatileBackupInfo` raises a
-        :exc:`NotImplementedError`.
+        :exc:`ValueError` if attempted to be saved to a file.
         """
         server = build_mocked_server()
         base_directory = "/fake/path/"
@@ -1641,7 +1663,17 @@ class TestVolatileBackupInfo:
             base_directory=base_directory,
             backup_id=backup_id,
         )
+        # Case 1: without passing any parameter (saves to file by default)
         with pytest.raises(
-            NotImplementedError, match="The save method is not implemented."
+            ValueError, match="VolatileBackupInfo does not support saving to a file"
         ):
             volatile_backup_info.save()
+
+        # Case 2: explicitly passing filename
+        with pytest.raises(
+            ValueError, match="VolatileBackupInfo does not support saving to a file"
+        ):
+            volatile_backup_info.save(filename="/path/to/some/file")
+
+        # Case 3: passing to file-like object works
+        volatile_backup_info.save(file_object=io.BytesIO())
